=== modified file 'Makefile'
--- Makefile	2011-10-15 16:48:03 +0000
+++ Makefile	2011-11-24 21:12:35 +0000
@@ -250,7 +250,7 @@
 	@echo "###################################################################"
 	./plugin-runner --plugin-dir=plugins.d \
 		--config-file=plugin-runner.conf \
-		--options-for=mandos-client:--seckey=keydir/seckey.txt,--pubkey=keydir/pubkey.txt \
+		--options-for=mandos-client:--seckey=keydir/seckey.txt,--pubkey=keydir/pubkey.txt,--network-hook-dir=network-hooks.d \
 		$(CLIENTARGS)
 
 # Used by run-client

=== added directory 'network-hooks.d'
=== added file 'network-hooks.d/bridge'
--- network-hooks.d/bridge	1970-01-01 00:00:00 +0000
+++ network-hooks.d/bridge	2011-11-24 20:15:24 +0000
@@ -0,0 +1,61 @@
+#!/bin/sh
+#
+# This is an example of a Mandos client network hook.  This hook
+# brings up a bridge interface as specified in a separate
+# configuration file.  To be used, this file and any needed
+# configuration file(s) should be copied into the
+# /etc/mandos/network-hooks.d directory.
+# 
+# Copying and distribution of this file, with or without modification,
+# are permitted in any medium without royalty provided the copyright
+# notice and this notice are preserved.  This file is offered as-is,
+# without any warranty.
+
+set -e
+
+CONFIG="$MANDOSNETHOOKDIR/bridge.conf"
+
+# Read config file, which must set "BRIDGE", "PORTS", and optionally
+# "IPADDRS" and "ROUTES".
+if [ -e "$CONFIG" ]; then
+    . "$CONFIG"
+fi
+
+if [ -z "$BRIDGE" -o -z "$PORTS" ]; then
+    exit
+fi
+
+if [ -n "$DEVICE" -a "$DEVICE" != "$BRIDGE" ]; then
+    exit
+fi
+
+case "$1" in
+    start)
+	brctl addbr "$BRIDGE"
+	for port in $PORTS; do
+	    brctl addif "$BRIDGE" "$port"
+	done
+	ip link set up "$BRIDGE"
+	if [ -n "$IPADDRS" ]; then
+            for ipaddr in $IPADDRS; do
+		ip addr add "$ipaddr" dev "$BRIDGE"
+	    done
+	fi
+	if [ -n "$ROUTES" ]; then
+            for route in $ROUTES; do
+		ip route add "$route" dev "$BRIDGE"
+	    done
+	fi
+	;;
+    stop)
+	ip link set down "$BRIDGE"
+	for port in $PORTS; do
+	    brctl delif "$BRIDGE" "$port"
+	done
+	brctl delbr "$BRIDGE"
+	;;
+    files)
+	echo /bin/ip
+	echo /usr/bin/brctl
+	;;
+esac

=== added file 'network-hooks.d/bridge.conf'
--- network-hooks.d/bridge.conf	1970-01-01 00:00:00 +0000
+++ network-hooks.d/bridge.conf	2011-11-24 20:15:24 +0000
@@ -0,0 +1,11 @@
+## Required
+
+#BRIDGE=br0
+
+#PORTS="eth0 eth1"
+
+## Optional
+
+#IPADDRS="192.0.2.3/24 2001:DB8::aede:48ff:fe71:f6f2/32"
+
+#ROUTES="192.0.2.0/24 2001:DB8::/32"

=== modified file 'plugin-runner.c'
--- plugin-runner.c	2011-10-09 12:32:13 +0000
+++ plugin-runner.c	2011-11-24 21:12:35 +0000
@@ -742,7 +742,7 @@
     }
   }
   
-  {
+  if(getuid() == 0){
     /* Work around Debian bug #633582:
        <http://bugs.debian.org/633582> */
     int plugindir_fd = open(/* plugindir or */ PDIR, O_RDONLY);

=== modified file 'plugins.d/mandos-client.c'
--- plugins.d/mandos-client.c	2011-11-14 20:55:32 +0000
+++ plugins.d/mandos-client.c	2011-11-24 21:12:35 +0000
@@ -1388,13 +1388,17 @@
   } else {
     int devnull = open("/dev/null", O_RDONLY);
     for(int i = 0; i < numhooks; i++){
-      direntry = direntries[0];
+      direntry = direntries[i];
       char *fullname = NULL;
       ret = asprintf(&fullname, "%s/%s", hookdir, direntry->d_name);
       if(ret < 0){
 	perror_plus("asprintf");
 	continue;
       }
+      if(debug){
+	fprintf_plus(stderr, "Running network hook \"%s\"\n",
+		     direntry->d_name);
+      }
       pid_t hook_pid = fork();
       if(hook_pid == 0){
 	/* Child */
@@ -1669,7 +1673,7 @@
     }
   }
     
-  {
+  if(getuid() == 0){
     /* Work around Debian bug #633582:
        <http://bugs.debian.org/633582> */
     struct stat st;
@@ -1733,20 +1737,24 @@
   
   /* Run network hooks */
   {
-    /* Re-raise priviliges */
-    errno = 0;
-    ret = seteuid(0);
-    if(ret == -1){
-      perror_plus("seteuid");
+    if(getuid() == 0){
+      /* Re-raise priviliges */
+      errno = 0;
+      ret = seteuid(0);
+      if(ret == -1){
+	perror_plus("seteuid");
+      }
     }
     if(not run_network_hooks("start", interface, delay)){
       goto end;
     }
-    /* Lower privileges */
-    errno = 0;
-    ret = seteuid(uid);
-    if(ret == -1){
-      perror_plus("seteuid");
+    if(getuid() == 0){
+      /* Lower privileges */
+      errno = 0;
+      ret = seteuid(uid);
+      if(ret == -1){
+	perror_plus("seteuid");
+      }
     }
   }
   
@@ -2206,10 +2214,12 @@
   
   /* Re-raise priviliges */
   {
-    errno = 0;
-    ret = seteuid(0);
-    if(ret == -1){
-      perror_plus("seteuid");
+    if(getuid() == 0){
+      errno = 0;
+      ret = seteuid(0);
+      if(ret == -1){
+	perror_plus("seteuid");
+      }
     }
     /* Run network hooks */
     if(not run_network_hooks("stop", interface, delay)){
@@ -2234,11 +2244,13 @@
       }
     }
   }
-  /* Lower privileges permanently */
-  errno = 0;
-  ret = setuid(uid);
-  if(ret == -1){
-    perror_plus("setuid");
+  if(getuid() == 0){
+    /* Lower privileges permanently */
+    errno = 0;
+    ret = setuid(uid);
+    if(ret == -1){
+      perror_plus("setuid");
+    }
   }
   
   /* Removes the GPGME temp directory and all files inside */

